#!/usr/bin/env ruby

require "./params_reader.rb"

def complement_proc(el)
	if(el == "A") 
        	el = "T"
      	elsif(el == "T")
      		el = "A"
      	elsif(el == "G") 
       	 	el = "C"
      	elsif(el == "C") 
      		el = "G"
      	elsif(el == "a")
      		el = "t"
      	elsif(el == "t")
        	el = "a"
      	elsif(el == "g")
      		el = "c"
      	elsif(el == "c") 
      		el = "g"
      	else
        	el = el
      	end
      	return el
end  


def read_datsets_ids(in_file)
	return IO.readlines(in_file).collect{|e| e.strip}
end

def extract_TF_names(seq_files)
	tf_cond = []
	Dir.entries(seq_files).each do |file|
		if(file.scan("seq_") != [] and File.stat("#{seq_files}/#{file}").directory?() == false)
			tf_cond.push(file.split("seq_")[1].strip)
		end
	end
	return tf_cond 
end

# returns the invert-complement of a sequence of nucleotides
def invertAndComplSequence(seq)
  return seq.split(//).reverse.collect {|el| complement_proc(el)}.join
end

def num_unique_kmers(seq, k)
  kmers = {}
  (0..(seq.length-k)).each do |i|
    kmers[seq[i..(i+k-1)]]=1
  end
  return kmers.length
end

def estimate_seq_stats(seq, rev_compl_flag)

	length = seq.strip.gsub(/\$/, "").gsub(/X/, "").length
	k=5
	num_uniqe_5mers = num_unique_kmers(seq, k)
 
	mm0 = 	{
		"A" =>0,
		"T" =>0,
		"C" =>0,
		"G"=>0
		}

	mm1 = 	{
		"AA"=>0,"AG"=>0,"AC"=>0,"AT"=>0,
         	"GA"=>0,"GG"=>0,"GC"=>0,"GT"=>0,
         	"CA"=>0,"CG"=>0,"CC"=>0,"CT"=>0,
         	"TT"=>0,"TG"=>0,"TC"=>0,"TA"=>0
		}

	if(mm0[seq[0..0]] != nil)
		mm0[seq[0..0]] += 1
	end
	(1...seq.length).each do |i|
		e = seq[i..i]
		if(mm0[e] != nil)
			mm0[e] += 1
		end
		ee = seq[i-1..i]
		if(mm1[ee] != nil)
			mm1[ee] += 1
		end
	end

	stats = {
		"MM0"=>mm0, 
		"MM1"=>mm1, 
		"length"=> {"length" => length}, 
		"num_uniqe_5mers"=>{"num_uniqe_5mers"=>num_uniqe_5mers}
		}

	if(rev_compl_flag == true)
#		print "rev_compl_flag == true\n"
#		$stdin.gets
		stats = {
				"MM0" => {
					"T_or_A"=>mm0["T"],
					"C_or_G"=>mm0["C"]},
				"MM1" => {
					"AC_or_GT"=>mm1["AC"], "CA_or_TG"=>mm1["CA"],
					"TC_or_GA"=>mm1["TC"], "CT_or_AG"=>mm1["CT"],
					"TT_or_AA"=>mm1["TT"], "CC_or_GG"=>mm1["CC"],
					"CG"=>mm1["CG"]      , "GC"=>mm1["GC"],
					"AT"=>mm1["AT"]      , "TA"=>mm1["TA"]
					 },
				"length" => {"length" => length},
				"num_uniqe_5mers"=>{"num_uniqe_5mers"=>num_uniqe_5mers}
			}
	else
		stats["MM0"].each do |key, v|
			stats["MM0"][key] = sprintf("%4d", v.to_i / 2)
		end
		stats["MM1"].each do |key, v|
			stats["MM1"][key] = sprintf("%4d", v.to_i / 2)
		end
		stats["length"]["length"] = stats["length"]["length"] / 2
		stats["num_uniqe_5mers"]["num_uniqe_5mers"] = stats["num_uniqe_5mers"]["num_uniqe_5mers"] / 2
	end
  return stats
end
#seq = "CTTTTTTTCTTTXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX$CTTTTTTTCTTTXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX$"
#rev_compl_flag = false
#estimate_seq_stats(seq, rev_compl_flag)


def extract_seq_stats(sequence_file, rev_compl_flag)
	annot_arr = []
	col_names = []

	data=[]
	count = 0
	seq=""
#	print "sequence_file: #{sequence_file}\n"
	lines = IO.readlines(sequence_file)[0].split("$")
	lines.each do |l|
		seq<<l+"$"
		count += 1
		if(count % (lines.length / 20) == 0)
			out = (count / (lines.length / 20))*(100 / 20)
			print out , "% "
			$stdout.flush 
		end
		if(count % 2 == 0)
			stats = estimate_seq_stats(seq, rev_compl_flag)
			evidence_entry_index = count / 2 - 1
			if(annot_arr != [])
				(0...annot_arr[evidence_entry_index].length).each do |i|
					if(col_names[i] != "ClusterID")
						stats[col_names[i]] = {col_names[i]=>annot_arr[evidence_entry_index][i]}
					end
				end
			end
			data.push([stats, seq])
			seq=""  
		end
	end
	return data
end

def build_entry(stats, params)
	entry = []
	params.each do |k, v|
		if(v == true and stats[k] != nil)
			entry.push(stats[k].values)
		end
	end
	return entry.flatten
end

def build_entry_names(stats, params)
	entry = []
	params.each do |k, v|
		if(v == true and stats[k] != nil)
			entry.push(stats[k].keys)
		end
	end
	return entry
end

def build_out_file_name(out_file_core, params)
	out_file_modified = "#{out_file_core}/"
	first = true
	params.each do |k, v|
		if(v == true)
			if(first == false)
				out_file_modified<<"_"
			end
			first = false
			out_file_modified<<"#{k}"
		end
	end
  	return out_file_modified
end

def output_seq_stats(data, out_file, params)

	stats = data[0][0]
	entry_names = build_entry_names(stats, params)

	entry = build_entry(stats, params)
	num_entries = entry.length
	num_lines = data.length
	if(num_entries == 0)
		print "empty file!\n"
		return
	end

	out_file_modified = build_out_file_name(out_file, params) 

	out_file_entry_names = out_file_modified+".var_names"
	f_out = File.open(out_file_entry_names, "w")
	f_out.write("#{entry_names.join(" ")}\n")
	f_out.close()

	print "\nout_file = #{out_file_modified}\n"
	print "-----------------------\n"

	f_out = File.open(out_file_modified, "w")
	f_out.write("#{num_lines}\t#{num_entries}\n")
	data.each do |e|
		stats = e[0]
		entry = build_entry(stats, params)
		out_line = "#{entry.join(" ")}\n"
		f_out.write(out_line)
	end
	f_out.close()
end


def do_seq_stats(io_params, params_instance)
	seq_file_full_path = io_params["input_file"]
	data               = extract_seq_stats(seq_file_full_path, io_params["rev_compl_flag"])
	out_dir            = io_params["out_dir"]
	output_seq_stats(data, out_dir, params_instance)
end

def extract_all_combos(io_params, params, keys_arr, params_instance, active_key_index)

	if(active_key_index == keys_arr.length)
		count = 0
		all_selections_false = true
		params_instance.each do |k, v|
			if(v == true)
				all_selections_false = false
			end
			print "#{k} = #{v}"
			if(count == params_instance.length - 1)
				print "\n"
			else
				print ", "
			end	
			count += 1
		end
		if(all_selections_false == true)
			return
		end
		do_seq_stats(io_params, params_instance)
		return
	end
	params[keys_arr[active_key_index]].each do |v|
		params_instance[keys_arr[active_key_index]] = v
		extract_all_combos(io_params, params, keys_arr, params_instance, active_key_index + 1)
	end
end

def extract_io_params(pars)
	io_params = ParamsReader.read_params_file(pars["io_params_file"])
	if(io_params["strand_specific_analysis"] == nil)
		printf("ERROR: 'extract_seq_covariates.rb' (function extract_io_params) 'strand_specific_analysis' invalid or not initialized!\n")
		exit 
	end
	io_params["rev_compl_flag"] = true
	if(io_params["strand_specific_analysis"] == "yes")
		io_params["rev_compl_flag"] = false
		print "strand_specific_analysis = yes\n"
	else
		if(io_params["strand_specific_analysis"] == "no")
			io_params["rev_compl_flag"] = true
			print "strand_specific_analysis = no\n"
		else
			print "ERROR: 'extract_seq_covariates.rb', allowable values for 'strand_specific_analysis' {yes, no}!\n"
			exit 	
		end

	end
	if(io_params["path_covariates_files"] == nil)
		io_params["path_covariates_files"]="input/covariates"
	end
	if(io_params["path_seq_cERMIT_format"] == nil)
		io_params["path_seq_cERMIT_format"]="input/data/seq"
	end
	# root directory for all input files
	if(io_params["input_root_dir"] == nil)
		io_params["input_root_dir"] = "input"
	end

	return io_params
end

def extract_covariate_params(covariates_params_file)
	covariate_params = {}
	ParamsReader.read_params_file(covariates_params_file).each do |e|
		covariate_params[e[0]] = []
		e[1].split(",").each do |val|
			if(val == "true")
				covariate_params[e[0]].push(true)
			elsif(val == "false")
				covariate_params[e[0]].push(false)
			else
				print "ERROR parameter values must be set to 'true' or 'false' or 'true,false'\n"
				exit	
			end
		end
	end
	return covariate_params
end

def run(pars)

	if(pars["io_params_file"] == nil)
		default_parms_io_file="params/io.params"
		if(FileTest.exist?(default_parms_io_file) == true)
			pars["io_params_file"] = default_parms_io_file
		else
			printf("USAGE: extract_seq_covariates.rb io_params_file=<file_name>!\n")
			exit 
		end
	end

	io_params = extract_io_params(pars)
	if(io_params["covariates_params_file"] == nil)
		printf("ERROR: 'extract_seq_covariates.rb', covariates_params_file=<file_path> entry must be especified inside #{pars["io_params_file"]}!\n")
		exit 
	end
	covariate_params = extract_covariate_params(io_params["covariates_params_file"])

	input_root_dir                      = io_params["input_root_dir"]
	io_params["path_seq_cERMIT_format"] = "#{input_root_dir}/#{io_params["path_seq_cERMIT_format"]}"
	io_params["path_covariates_files"]  = "#{input_root_dir}/#{io_params["path_covariates_files"]}"

	tf_cond=""
	if(io_params["dataset_ids_file"] != nil)
		tf_cond  = read_datsets_ids(io_params["dataset_ids_file"])
	else
		tf_cond  = extract_TF_names(io_params["path_seq_cERMIT_format"])
	end

#	print "tf_cond.inspect: #{tf_cond.inspect}\n"

#	print "#{io_params["path_seq_cERMIT_format"]}\n"

	ind = 0
	Dir.entries(io_params["path_seq_cERMIT_format"]).each do |seq_file_cERMIT_format|
		if(seq_file_cERMIT_format.scan("seq_") != [])
			tf_name = seq_file_cERMIT_format.split("seq_")[1]

			cov_out_dir="#{io_params["path_covariates_files"]}/#{tf_name}"
#			print "cov_out_dir = #{cov_out_dir}\n"
			if(tf_cond.include?(tf_name) == false or (FileTest.exist?(cov_out_dir) == true) )
				next
			end
			ind += 1
			print "#{ind}: #{tf_name}\n"

			io_params["input_file"] = "#{io_params["path_seq_cERMIT_format"]}/#{seq_file_cERMIT_format}"
			sub_dir                 = seq_file_cERMIT_format.split("seq_")[1]
			original_out_dir        = "#{io_params["path_covariates_files"]}/#{sub_dir}"
			io_params["out_dir"]    = "#{original_out_dir}_tmp"

			if(FileTest.exist?(io_params["out_dir"]) == false)
				system("mkdir --parents #{io_params["out_dir"]}")
			end

#			print "out_dir: #{io_params["out_dir"]}\n"
#			print "sequence regions file: #{io_params["input_file"] }\n"
			print "================================================================\n"
			active_key_index = 0
			params_instance  = {}
			extract_all_combos(io_params, covariate_params, covariate_params.keys, params_instance, active_key_index)
			system("mv #{io_params["out_dir"]} #{original_out_dir}")
		end
	end
	
	print "extracting covariates from #{tf_cond.length} data sets...done\n"

end

#########################
# command-line parameters
#########################

def parse_ARGV()
  pars = {}
  ARGV.each do |a|
    pars[a.split("=")[0]] = a.split("=")[1]
  end
  return pars
end
pars = parse_ARGV()

########
# RUN
########
run(pars)
########
# RUN
########

