#!/usr/bin/env ruby

require "./params_reader.rb"

def read_datsets_ids(in_file)
	return IO.readlines(in_file).collect{|e| e.strip}
end

def extract_TF_names(seq_files)
	tf_cond = []
	Dir.entries(seq_files).each do |file|
		if(file.scan("seq_") != [] and File.stat("#{seq_files}/#{file}").directory?() == false)
			tf_cond.push(file.split("seq_")[1].strip)
		end
	end
	return tf_cond 
end

def construct_evidence_file_list_entry(path_evidence_files, path_pssm_files, cond)
	pssm_entry=""
	if(File.exists?(path_pssm_files) == true)
		pssm_entry="#{path_pssm_files}/pssm_#{cond.split("_")[0]}.txt"
	end
	return "#{path_evidence_files}/evidence_#{cond}\t#{pssm_entry}"
end

def construct_seq_file_list_entry(path_seq_files, cond)
	seq_file_list_entry = "#{path_seq_files}/seq_#{cond}"
	Dir.entries(path_seq_files).each do |entry|
		if(cond.scan("cons") != [] and entry.scan(cond) != [] and entry != "seq_#{cond}")
			seq_file_list_entry<<"\t#{path_seq_files}/#{entry}"
		end
	end
	return seq_file_list_entry
end

def construct_covariates_file_list_entry(path_covariates_files, cond, covariates_type)
	return "#{path_covariates_files}/#{cond}/#{covariates_type}"
end

def add_entry_to_defaults_hash(defaults, io_params, entry, input_root_dir)
	key=entry[0]
	default_value=entry[1]
	defaults[key] = default_value
	if(io_params[key] != nil)
		defaults[key] = "#{input_root_dir}/#{io_params[key]}"
	end
end

def get_file_list_files_default_vals(io_params)
	defaults = {}
	# root directory for all input files
	if(io_params["input_root_dir"] == nil)
		io_params["input_root_dir"] = "input"
	end
	[
		["path_covariates_files" ,"#{io_params["input_root_dir"]}/covariates"        ],
		["subdir_cov"            ,"#{io_params["input_root_dir"]}/cov_file_list"     ],

		["path_evidence_files"   ,"#{io_params["input_root_dir"]}/data/evidence"     ],
		["subdir_evidence"       ,"#{io_params["input_root_dir"]}/evidence_file_list"],

		["path_seq_cERMIT_format","#{io_params["input_root_dir"]}/data/seq"          ],
		["subdir_seq"            ,"#{io_params["input_root_dir"]}/seq_file_list"     ],

		["path_pssm_files"       ,"#{io_params["input_root_dir"]}/pssms"             ]
	].each do |e|
		add_entry_to_defaults_hash(defaults, io_params, e, io_params["input_root_dir"])
	end
	defaults["covariates_to_use"] = "MM1"
	if(io_params["covariates_to_use"] != nil)
		defaults["covariates_to_use"] = io_params["covariates_to_use"]
	end
	return defaults
end

def extract_io_params(pars)
	if(pars["io_params_file"] == nil)
		default_parms_io_file="params/io.params"
		if(FileTest.exist?(default_parms_io_file) == true)
			pars["io_params_file"] = default_parms_io_file
		else
			print "USAGE: ./fasta_format_reader.rb io_params_file=<params_file_name>\n"
			exit(1)
		end
	end
	io_params = ParamsReader.read_params_file(pars["io_params_file"])
	return io_params
end

def run(pars)

	io_params                    = extract_io_params(pars)
	file_list_files_default_vals = get_file_list_files_default_vals(io_params)

	path_pssm_files       = file_list_files_default_vals["path_pssm_files"]
	path_evidence_files   = file_list_files_default_vals["path_evidence_files"]
	subdir_evidence       = file_list_files_default_vals["subdir_evidence"]
	path_seq_files        = file_list_files_default_vals["path_seq_cERMIT_format"]
	subdir_seq            = file_list_files_default_vals["subdir_seq"]
	path_covariates_files = file_list_files_default_vals["path_covariates_files"]
	subdir_cov            = file_list_files_default_vals["subdir_cov"]
	covariates_type       = file_list_files_default_vals["covariates_to_use"]

	if(File.exists?("#{subdir_cov}") == false)
		Dir.mkdir("#{subdir_cov}")
	end
	if(File.exists?("#{subdir_evidence}") == false)
		Dir.mkdir("#{subdir_evidence}")
	end
	if(File.exists?("#{subdir_seq}") == false)
		Dir.mkdir("#{subdir_seq}")
	end

	if(io_params["dataset_ids_file"] != nil)
		tf_cond  = read_datsets_ids(io_params["dataset_ids_file"])
	else
		tf_cond  = extract_TF_names(path_seq_files)
	end

	print "Creating auxiliary folders:\n"
	tf_cond.each do |cond|
		# construct 'evidence_file_list'
		f_out_evidence_list = File.open("#{subdir_evidence}/evidence_file_list_#{cond}", "w")
		entry_evidence = construct_evidence_file_list_entry(path_evidence_files, path_pssm_files, cond)
		f_out_evidence_list.write(entry_evidence)
		f_out_evidence_list.close()
	
		# construct 'sequence_file_list'
		f_out_seq_list = File.open("#{subdir_seq}/sequence_file_list_#{cond}", "w")
		entry_seq = construct_seq_file_list_entry(path_seq_files, cond)
		f_out_seq_list.write(entry_seq)
		f_out_seq_list.close()
	
		# construct 'covariates_file_list'
		f_out_covariates_list = File.open("#{subdir_cov}/covariates_file_list_#{cond}", "w")
		entry_cov = construct_covariates_file_list_entry(path_covariates_files, cond, covariates_type)
		f_out_covariates_list.write(entry_cov)
		f_out_covariates_list.close()
		print "#{subdir_evidence}/evidence_file_list_#{cond}\n#{subdir_seq}/sequence_file_list_#{cond}\n#{subdir_cov}/covariates_file_list_#{cond}\n==========\n"
	end

end

def parse_ARGV()
  pars = {}
  ARGV.each do |a|
    pars[a.split("=")[0]] = a.split("=")[1]
  end
  return pars
end
pars = parse_ARGV()

########
# RUN
########
run(pars)
########
# RUN
########
