#!/usr/bin/env ruby

require "params_reader.rb"

def read_dataset_sim_scores(out_dir_dataset_run)
	sim_scores_all = {}
	Dir.entries(out_dir_dataset_run).each do |entry|
		full_path = "#{out_dir_dataset_run}/#{entry}"
		if(FileTest.file?(full_path) == true and entry.scan("max_sim_") != [])
			sim_score  = IO.readlines(full_path)[0].split("\s")[1].strip.to_f
			dataset_id = IO.readlines(full_path)[0].split("\s")[0].strip
			out_entry = "#{dataset_id}\t#{sim_score}\n"
			sim_scores_all[dataset_id] = sim_score
		end
	end
	return sim_scores_all
end

def load_results_into_single_table(root_summary_dir, out_dir_prefix, tmp_suffix)
	dataset_results = []
#	Dir.entries(".").each do |entry|
	Dir.entries(root_summary_dir).each do |entry|
		full_path = "#{root_summary_dir}/#{entry}"
#		print "full_path: #{full_path}\n"
		if(FileTest.file?(full_path) == false and entry.scan("#{out_dir_prefix}") != [] and entry.scan(tmp_suffix) == [])
			sim_scores_all = read_dataset_sim_scores(full_path)
			dataset_results.push([entry, sim_scores_all])
		end
	end
	return dataset_results
end


def plot_results(filename_R_script, input_file, out_file_pdf, sim_threshold, upper_bound)
	command = "R < #{filename_R_script} --slave --vanilla --args #{input_file} #{out_file_pdf} #{sim_threshold} #{upper_bound}";
	system(command)
end


def generate_dataset_plots(filename_R_script, out_dir_dataset_run, sim_threshold)
	file_in_all   = "#{out_dir_dataset_run}/summary_all_sim_scores"
	file_out_all  = "#{out_dir_dataset_run}/all_sim_scores.pdf"
	plot_results(filename_R_script, file_in_all, file_out_all, sim_threshold)

	file_in_succ  = "#{out_dir_dataset_run}/summary_succ_sim_scores"
	file_out_succ = "#{out_dir_dataset_run}/succ_sim_scores.pdf"
	plot_results(filename_R_script, file_in_succ, file_out_succ, sim_threshold)

	file_in_fail  = "#{out_dir_dataset_run}/summary_fail_sim_scores"
	file_out_fail = "#{out_dir_dataset_run}/fail_sim_scores.pdf"
	plot_results(filename_R_script, file_in_fail, file_out_fail, sim_threshold)
end

def extract_io_params(pars)
	if(pars["io_params_file"] == nil)
		pars["io_params_file"] = "params/io.params"
	end
	io_params = ParamsReader.read_params_file(pars["io_params_file"])
	return io_params
end


def parse_input_params(pars)

	io_params = extract_io_params(pars)
	if(io_params["filename_R_script"] == nil)
		io_params["filename_R_script"]="plot_results.R"
	end
	if(io_params["tmp_suffix"] == nil)
		io_params["tmp_suffix"]="tmp"
	end
	if(io_params["out_dir_prefix"] == nil)
		io_params["out_dir_prefix"]="out"
	end
	return io_params
end

def extract_TFs_with_complete_set_of_redictions(dataset_results)
	count_hash = {}
	dataset_results.each do |d1|
		dataset_id = d1[0]
		d = d1[1]
		d.each do |k, v|
			if(count_hash.keys.include?(k) == false)
				count_hash[k] = 0
			end
			count_hash[k] += 1
		end	
	end
	max_count = count_hash.values.max

	tf_names_with_no_missing_data = []
	count_hash.each do |k, v|
		if(v == max_count)
			tf_names_with_no_missing_data.push(k)
		else
			print "incomplete entry: #{k}\n"
		end
	end
	return tf_names_with_no_missing_data
end


def extract_id(features_arr, long_id)
	id = ""
	features_arr.each do |f|
		val = long_id.split("#{f}_")[1].split("_")[0]
		id<<"#{f}=#{val}_"
	end
	arr = id.split("")
	arr.pop
	id = arr.join("")
	return id
end

def extract_dataset_ids(dataset_results, features_arr)
	ids = []
	ind = 0
	dataset_results.each do |d|
		id = extract_id(features_arr, d[0])
		ids.push([ind, id])
		ind += 1
	end
	ids = ids.sort{|a,b| a[1].split("=").last.to_f <=> b[1].split("=").last.to_f}
	return ids
end

#  Add methods to Enumerable, which makes them available to Array
module Enumerable
 
  #  sum of an array of numbers
  def sum
    return self.inject(0){|acc,i|acc +i}
  end
 
  #  average of an array of numbers
  def average
    return self.sum/self.length.to_f
  end
 
  #  variance of an array of numbers
  def sample_variance
    avg=self.average
    sum=self.inject(0){|acc,i|acc +(i-avg)**2}
    return(1/self.length.to_f*sum)
  end
 
  #  standard deviation of an array of numbers
  def standard_deviation
    return Math.sqrt(self.sample_variance)
  end
 
end  #  module Enumerable


def find_dir(root_dir, dir_substring)
	Dir.entries(root_dir).each do |entry|
		if(FileTest.file?(entry) == false and entry.scan("#{dir_substring}") != [])
			return entry
		end
	end
	return ""
end

def generate_results_matrix(dataset_results, tf_names_with_no_missing_data, root_summary_dir, out_dir_prefix, tmp_suffix)
	results_matrix = {}
	dataset_results.each do |d1|
		dataset_id = d1[0]
		d = d1[1]
		d.each do |k, v|
			if(tf_names_with_no_missing_data.include?(k) == true)
				if(results_matrix.keys.include?(k) == false)
					results_matrix[k] = []
				end
				results_matrix[k].push(v.to_f)
			end
		end
	end 
	return results_matrix
end

def run(pars)
	io_params                     = parse_input_params(pars)
	sim_threshold                 = io_params["sim_threshold"]
	out_dir_prefix                = io_params["out_dir_prefix"]
	tmp_suffix                    = io_params["tmp_suffix"]
#	root_summary_dir              = "."#find_dir(".", "summary")
	root_summary_dir              = io_params["result_dir"]
#	print "root_summary_dir = #{root_summary_dir}\n"
#	$stdin.gets	

	dataset_results               = load_results_into_single_table(root_summary_dir, out_dir_prefix, tmp_suffix)
#	print dataset_results.inspect, "\n"
#	$stdin.gets
	tf_names_with_no_missing_data = extract_TFs_with_complete_set_of_redictions(dataset_results)

	results_matrix = generate_results_matrix(dataset_results, tf_names_with_no_missing_data, root_summary_dir, out_dir_prefix, tmp_suffix)
	
	

	f_out = File.open("results_matrix", "w")
	features_arr = ["min_percent"]#, "bootstrap", "regression"]
	dataset_ids = extract_dataset_ids(dataset_results, features_arr)
	data_matrix_header = ""
	dataset_ids.each do |e|
		col_name="#{e[1].split("=")[1]}"
		print "col_name=#{col_name}\n"
		data_matrix_header<<"#{col_name}\t"
	end
	f_out.write("#{data_matrix_header}\n")
	f_out_tf_names = File.open("results_matrix.row_names", "w")

	tf_cond_names = results_matrix.keys
	results_matrix.sort{|a,b| b[1].average <=> a[1].average}.each do |k, v|
#	data_matrix.sort{|a,b| b[1].standard_deviation <=> a[1].standard_deviation}.each do |k, v|
		line = ""
		f_out_tf_names.write("#{k}\n")
		(0...v.length).each do |ind|
			i = dataset_ids[ind][0]
			line<<"#{v[i]}\t"	
		end
		f_out.write("#{line}\n")
	end
	f_out_tf_names.close()
	f_out.close()

	input_file = "results_matrix"
	out_file_pdf = "plot_results.pdf"
	filename_R_script = io_params["filename_R_script"]
	sim_threshold = 0.75
	upper_bound=156
	plot_results(filename_R_script, input_file, out_file_pdf, sim_threshold, upper_bound)
end


def parse_ARGV()
  pars = {}
  ARGV.each do |a|
    pars[a.split("=")[0]] = a.split("=")[1]
  end
  return pars
end
pars = parse_ARGV()

########
# RUN
########
run(pars)
########
# RUN
########

