#!/usr/bin/env ruby

require "./params_reader.rb"

class Fasta_reader
  def initialize()
  end

private
	@@complement_proc = lambda do |el|
		if(el == "A") 
			el = "T"
		elsif(el == "T")
		el = "A"
		elsif(el == "G") 
			el = "C"
		elsif(el == "C") 
		el = "G"
		elsif(el == "a")
		el = "t"
		elsif(el == "t")
			el = "a"
		elsif(el == "g")
		el = "c"
		elsif(el == "c") 
		el = "g"
		else
			el = el
		end
		return el
	end  
	
	def Fasta_reader.invertAndComplSequence(seq)
		return seq.split(//).reverse.collect {|el| @@complement_proc.call(el)}.join
	end
	
	def Fasta_reader.pad_X(seq, max_len)
		len = max_len - seq.length
		out_seq = seq
		(0...len).each do |i|
		out_seq<<'X'
		end
		return out_seq
	end

	def Fasta_reader.find_max_len(data)
		max_len = 0
		data.each do |gid, d|
			d["sequence"].each do |id, seq|
				len = seq.strip.length
				if(max_len < len)
					max_len = len
				end
			end
		end
		return max_len
	end

public
	def Fasta_reader.pass_filter(seq, seq_length_threshold)
		#    seq_length_threshold = 100000
		return ( (seq.length <= seq_length_threshold) and (/[^XAGCTagct]/.match(seq) == nil))
	end

	def Fasta_reader.add_padding_and_rev_compl_seq(data, include_rev_compl)
		ind = 1
		max_len = Fasta_reader.find_max_len(data)
		percent_stride=5
		num_seq = data.length
		divisor = 100 / percent_stride
		data.each do |gid, d|
			ind +=1
			if(ind % (num_seq / divisor) == 0)
				print "#{ (ind / (num_seq / divisor)) * percent_stride }% "
				$stdout.flush
			end
			d["sequence"].each do |id, seq|
				padded_seq = Fasta_reader.pad_X(d["sequence"][id], max_len)
				
				# add rev compl
				d["sequence"][id] = padded_seq+"$"
				if(include_rev_compl == true) 
					d["sequence"][id]<<Fasta_reader.invertAndComplSequence(padded_seq)+"$"
				else
					d["sequence"][id]<<padded_seq+"$"
				end
			end	
		end
		print "\n"
		return data
	end

	def Fasta_reader.is_multi_fasta?(file, species_id_delim)
		File.open(file, "r").each do |l|
			if(l.scan(/^>/) == [] and l.scan("#{species_id_delim}") != [])
				return true
			else
				return false
			end
		end		
	end

	def Fasta_reader.parse_multi_fasta_file(file, species_id_delim, seq_length_threshold)
		data = {}
		failed_arr = []
		gene_arr = IO.read(file).split(">")
		gene_arr[1..-1].each do |entry|
			entry =~ /^([\S]+)\|([\S]+)\n([\S|\s]+)/
			id = $1.strip
			binding_ev = $2.strip
			seq_hash= {}

			$3.split("\n").each do |e|
				arr = e.split(species_id_delim)
				if(arr.length == 1)
					seq_hash["reference"] = e.gsub(/\s/, '').strip.upcase
				else
					species_id = arr[0]
					seq = arr[1].strip.upcase
					seq_hash[species_id] = seq
				end
			end
			entry.split("\n").each do |line|
				arr = line.split("#{species_id_delim}")
				if(line.scan(">") != [] and arr.length == 2)
					species_id=arr[0]
					species_seq=arr[1].strip.upcase
					seq_hash[species_id]=species_seq
				end
			end
			failed_flag=false
			seq_hash.each do |species_id, species_seq|
				if(Fasta_reader.pass_filter(species_seq, seq_length_threshold) == false)
					failed_arr.push(entry)
					failed_flag = true
					break
				end
			end
			if(failed_flag == false)
				data[id] = {"binding_evidence"=>binding_ev, "sequence"=>seq_hash}
			end
		end
		data_sorted = data.sort{|a,b| b[1]["binding_evidence"].to_f  <=> a[1]["binding_evidence"].to_f}
		return [data_sorted, failed_arr]
	end

	def Fasta_reader.write_to_file_cERMIT_format(data, out_seq_file, out_ids_file, include_rev_compl)
		data = Fasta_reader.add_padding_and_rev_compl_seq(data, include_rev_compl)
		species_ids = {}
		f_out_seq_arr = []
		f_out_evidence = File.open(out_ids_file, "w")
		ind=0
		data[0][1]["sequence"].each do |species_id, seq|
			out_seq_filename="#{out_seq_file}#{species_id}".split("reference")[0]
			species_ids[species_id] = ind
			ind += 1
			f_out_seq = File.open(out_seq_filename, "w")
			f_out_seq_arr.push(f_out_seq)
		end
		data.each do |gid, d|
			f_out_evidence.write("#{gid.to_s}\t#{d["binding_evidence"]}\n")
			species_ids.each do |s_id, index|
				f_out_seq_arr[index].write(d["sequence"][s_id])
			end
		end
		f_out_evidence.close()
		species_ids.each do |s_id, index|
			f_out_seq_arr[index].close()
		end
	end
end


def extract_io_params(pars)
	if(pars["io_params_file"] == nil)
		default_parms_io_file="params/io.params"
		if(FileTest.exist?(default_parms_io_file) == true)
			pars["io_params_file"] = default_parms_io_file
		else
			print "USAGE: ./fasta_format_reader.rb io_params_file=<params_file_name>\n"
			exit(1)
		end
	end
	io_params = ParamsReader.read_params_file(pars["io_params_file"])

	# sequence length upper bound
	if(io_params["seq_length_threshold"] == nil)
		io_params["seq_length_threshold"] = 10000
	end
	# root directory for all input files
	if(io_params["input_root_dir"] == nil)
		io_params["input_root_dir"] = "input"
	end
	# input directory of the fasta files to be converted in to cERMIT format input
	if(io_params["fasta_input_dir"] == nil)
		io_params["fasta_input_dir"] = "#{io_params["input_root_dir"]}/data/fasta"
	end
	# output directory of the cERMIT format sequence files
	if(io_params["path_seq_cERMIT_format"] == nil)
		io_params["path_seq_cERMIT_format"] = "#{io_params["input_root_dir"]}/#{io_params["path_seq_cERMIT_format"]}"
	end
	# output directory of the cERMIT format evidence files
	if(io_params["path_evidence_files"] == nil)
		io_params["path_evidence_files"] = "#{io_params["input_root_dir"]}/data/evidence"
	end
	# DNA=>strand_specific_analysis=no, RNA=>strand_specific_analysis=yes
	if(io_params["strand_specific_analysis"] != "yes" and io_params["strand_specific_analysis"] != "no")
		print "ERROR: 'fasta_format_reader.rb', allowable values for 'strand_specific_analysis' {yes, no}!\n"
		exit 
	end
	# delimiter used separate the species id from its corresponding sequence (inpside multi-fasta file)
	if(io_params["species_id_delim"] == nil)
		io_params["species_id_delim"] = ":"
	end

	return io_params
end

def read_datsets_ids(in_file)
	return IO.readlines(in_file).collect{|e| e.strip}
end

def extract_TF_names_from_fasta_input_dir(fasta_input_dir)
	tf_cond = []
	Dir.entries(fasta_input_dir).each do |file|
		if(file.scan(".fasta") != [] and File.stat("#{fasta_input_dir}/#{file}").directory?() == false)
			tf_cond.push(file.split(".fasta")[0].strip)
		end
	end
	return tf_cond 
end

def run(pars)
	io_params = extract_io_params(pars)

	seq_length_threshold = io_params["seq_length_threshold"].to_i
	input_root_dir       = io_params["input_root_dir"]
	fasta_input_dir      = "#{input_root_dir}/#{io_params["fasta_input_dir"]}"
	out_seq_dir          = "#{input_root_dir}/#{io_params["path_seq_cERMIT_format"]}"
	out_evidence_dir     = "#{input_root_dir}/#{io_params["path_evidence_files"]}"
	species_id_delim     = io_params["species_id_delim"]

	include_rev_compl = true
	if(io_params["strand_specific_analysis"]=="yes")
		include_rev_compl = false
	end
	if(FileTest.exist?(out_seq_dir) == false)
		system("mkdir --parents #{out_seq_dir}")
	end
	if(FileTest.exist?(out_evidence_dir) == false)
		system("mkdir --parents #{out_evidence_dir}")
	end

	tf_cond=""
	if(io_params["dataset_ids_file"] != nil)
		tf_cond  = read_datsets_ids(io_params["dataset_ids_file"])
	else
		tf_cond  = extract_TF_names_from_fasta_input_dir(fasta_input_dir)
	end

#	print "#{tf_cond.inspect}\n"
#	$stdin.gets

	ind = 0
	Dir.entries(fasta_input_dir).each do |fasta_file|
		if(fasta_file.scan(".fasta") != [])
			fasta_file="#{fasta_input_dir}/#{fasta_file}"
			dataset_name = fasta_file.split("/").last.split(".fasta")[0]
			out_seq_file = "#{out_seq_dir}/seq_#{dataset_name}"
			out_evidence_file = "#{out_evidence_dir}/evidence_#{dataset_name}"
			
#			print "out_seq_file: #{out_seq_file}\n"
#			print "out_evidence_file: #{out_evidence_file}\n"
#			print "FileTest.exist?(out_seq_file) = #{FileTest.exist?(out_seq_file)}\n"
#			print "FileTest.exist?(out_evidence_file) = #{FileTest.exist?(out_evidence_file)}\n"
			if(tf_cond.include?(dataset_name) == false or ((FileTest.exist?(out_seq_file) == true) and (FileTest.exist?(out_evidence_file) == true)) )
				next
			end

			ind += 1
			print "#{ind}: #{dataset_name}\n"
			
			data, failed_arr = Fasta_reader.parse_multi_fasta_file(fasta_file, species_id_delim, seq_length_threshold)
			print "number of read sequences: #{data.length}\n"
			print "failed (> #{seq_length_threshold}bp): #{failed_arr.length}\n"

#			out_evidence_file_tmp = "#{out_evidence_file}_tmp";
#			out_seq_file_tmp = "#{out_seq_file}_tmp"
			Fasta_reader.write_to_file_cERMIT_format(data, out_seq_file, out_evidence_file, include_rev_compl)
#			system("mv #{out_evidence_file_tmp} #{out_evidence_file}")
#			system("mv #{out_seq_file_tmp} #{out_seq_file}")
		end
	end
	print "reading FASTA input from #{tf_cond.length} data sets...done\n"
end

#########################
# command-line parameters
#########################

def parse_ARGV()
  pars = {}
  ARGV.each do |a|
    pars[a.split("=")[0]] = a.split("=")[1]
  end
  return pars
end
pars = parse_ARGV()

#############
# run command
#############
run(pars)
#./fasta_format_reader.rb io_params_file=params/io.params
#############
# run command
#############
